@@ -692,7 +692,7 @@ def warnmetadataloss(repo, local, other):
     public content-divergence"""
 
     # needtowarn: aspects where we need to warn user
-    needtowarn = [b'branch', b'topic', b'close']
+    needtowarn = [b'branch', b'close']
     aspects = set()
     localextra = local.extra()
     otherextra = other.extra()
@@ -959,18 +959,9 @@ def _relocate(repo, orig, dest, evolvestate, pctx=None, keepbranch=False,
     _finalizerelocate(repo, orig, dest, nodenew, tr, category, evolvestate)
     return nodenew
 
-# This is copied from hgext.rebase._savegraft()
-def _savegraft(ctx, extra):
-    s = ctx.extra().get(b'source', None)
-    if s is not None:
-        extra[b'source'] = s
-    s = ctx.extra().get(b'intermediate-source', None)
-    if s is not None:
-        extra[b'intermediate-source'] = s
-
 def _relocatecommit(repo, orig, dest, pctx, keepbranch, commitmsg, update):
     extra = {}
-    _savegraft(orig, extra)
+    compat.preserve_extras_on_rebase(orig, extra)
     extra[b'rebase_source'] = orig.hex()
     targetphase = max(orig.phase(), phases.draft)
     configoverrides = {
@@ -2201,7 +2192,7 @@ def _completerelocation(ui, repo, evolvestate):
     ctx = orig
 
     extra = {}
-    _savegraft(ctx, extra)
+    compat.preserve_extras_on_rebase(orig, extra)
     extra[b'rebase_source'] = orig.hex()
 
     user = ctx.user()
