@@ -49,14 +49,29 @@ def nonpublicphaseroots(repo):
     )
 
 def overridecommitstatus(overridefn):
-    if r'tip' in cmdutil.commitstatus.__code__.co_varnames:
+    code = cmdutil.commitstatus.__code__
+    if r'opts' in code.co_varnames[code.co_argcount:]:
+        # commitstatus(..., **opts)
         extensions.wrapfunction(cmdutil, 'commitstatus', overridefn)
+    elif r'tip' in code.co_varnames:
+        # hg <= 6.5 (489268c8ee7e)
+        def _override(orig, repo, node, branch, bheads=None, tip=None, opts=None):
+            def _orig(repo, node, branch, bheads=None, tip=None, **opts):
+                opts = pycompat.byteskwargs(opts)
+                return orig(repo, node, branch, bheads=bheads, tip=tip, opts=opts)
+            opts = pycompat.strkwargs(opts)
+            return overridefn(_orig, repo, node, branch, bheads=bheads, tip=tip, **opts)
+        extensions.wrapfunction(cmdutil, 'commitstatus', _override)
     else:
         # hg <= 5.6 (976b26bdd0d8)
         def _override(orig, repo, node, branch, bheads=None, opts=None):
-            def _orig(repo, node, branch, bheads=None, tip=None, opts=None):
+            def _orig(repo, node, branch, bheads=None, **opts):
+                opts = pycompat.byteskwargs(opts)
                 return orig(repo, node, branch, bheads=bheads, opts=opts)
-            return overridefn(_orig, repo, node, branch, bheads=bheads, tip=None, opts=opts)
+            if opts is None:
+                opts = {}
+            opts = pycompat.strkwargs(opts)
+            return overridefn(_orig, repo, node, branch, bheads=bheads, **opts)
         extensions.wrapfunction(cmdutil, 'commitstatus', _override)
 
 if util.safehasattr(error, 'InputError'):
